<!DOCTYPE html>
<html lang="en">

<head>
  <title>{{test_name}}</title>
</head>

<body>
  <h1>Results: {{test_name}}</h1>
  <hr>
  <!-- rank accuracy -->
  <h3>Ranking accuracy</h3>
  {% for i in range(rank_accu | length) %}
  <div>
    Top {{rank_accu[i][0]}}: {{rank_accu[i][1]}}
  </div>
  {% endfor %}

  <hr>
  <h3>Ranking visualization</h3>
  {% for i in range(query_fns|length) %}
  <div style="width:25%;">
    <h3>query {{i}}</h3>
    <img src="{{query_fns[i]}}" alt="" style="width: 100%; border-style: solid; border-width: 2px;">
    <h4 style="text-align: center">Class: {{query_labels[i]}}</h4>
  </div>
  <h3>top results</h3>
  {% for j in range(db_fns[i]|length) %}
  <div style="display: inline-block; max-width: 25%;">
    <img src="{{db_fns[i][j]}}" alt="" style="width: 100%; border-style: solid; border-width: 3px;">
    <h4 style="text-align: center;">Class: {{db_labels[i][j]}}</h4>
  </div>
  {% endfor %}
  <hr> {% endfor %}
</body>

</html>