"""Google vision api manager.
"""


class GoogleVisionAPI(object):
  """Class to manage google cloud vision api.
  """
  client = None

  def __init__(self):
    pass

  def label_img(self, img_base64):
    """Label image with general lables.
    """
    pass
